/*
 * This file is part of unity-2d
 *
 * Copyright 2011 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 1.0
import "../common"
import "../common/utils.js" as Utils
import "../common/units.js" as Units

FocusScope {
    id: preview
    property string uri
    property int category

    property variant nfo
    property variant buttons
    property variant previewImage: previewImage
    property int buttonWidth: (previewImage.width - Units.tvPx(64)) / 3

    KeyNavigation.right: infoFlickable.visibleArea.heightRatio < 1.0 ? infoFlickable : buttons

    nfo: VideoInfo {
        uri: preview.uri
    }

    TextCustom {
        id: title
        anchors.left: parent.left
        anchors.top: parent.top
        fontSize: "xx-large"
        color: "white"
        text: nfo.video ? nfo.video.title : ""
    }

    TextCustom {
        id: titleExtraInfo
        anchors.left: title.right
        anchors.leftMargin: Units.tvPx(35)
        anchors.top: parent.top
        fontSize: "xx-large"
        color: "white"
        opacity: 0.5
        property string baseTitleMovie: u2d.tr('(%1)')
        property string baseTitleTV: u2d.tr('(%1 %2x%3)')
        text: {
            if (nfo.video && nfo.isMovie) {
                return baseTitleMovie.arg(nfo.video.year)
            } else if (nfo.video && nfo.tvShow) {
                return baseTitleTV.arg(nfo.tvShow.title).arg(nfo.video.season).arg(nfo.video.episode)
            } else {
                return ""
            }
        }
    }

    BorderImage {
        id: previewImageDropShadow
        source: "artwork/video_preview_shadow.sci"

        anchors.fill: previewImage
        anchors.margins: -Units.tvPx(15)

        smooth: true
        asynchronous: true
        opacity: 0.4
    }

    PreviewImageBox {
        id: previewImage
        anchors.left: parent.left
        anchors.right: info.left
        anchors.rightMargin: Units.tvPx(35)
        anchors.top: title.bottom
        anchors.topMargin: Units.tvPx(50)
        anchors.bottom: buttons.top
        anchors.bottomMargin: Units.tvPx(60)
        margin: Units.tvPx(3)

        color: "black"
        showBackgroundImage: false
        showBlackBoundary: false
        source: Utils.removeExt(uri) + ((nfo.isMovie) ? "-fanart.jpg" : ".tbn")
    }

    Item {
        id: info
        anchors.top: title.bottom
        anchors.topMargin: Units.tvPx(50)
        anchors.right: parent.right
        anchors.rightMargin: Units.tvPx(25)
        anchors.bottom: parent.bottom
        anchors.bottomMargin: Units.tvPx(50)
        width: Units.tvPx(500)

        KeyNavigation.left: buttons

        Flickable {
            id: infoFlickable
            clip: true
            anchors.fill: parent

            contentHeight: column.height

            Keys.onDownPressed: contentY = Math.min(contentHeight - height, contentY + 200)
            Keys.onUpPressed: contentY = Math.max(0, contentY - 200)

            Behavior on contentY { NumberAnimation { } }

            Column {
                id: column
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.rightMargin: Units.tvPx(25)
                spacing: Units.tvPx(35)

                RatingStars {
                    id: rating
                    rating: (nfo.video) ? nfo.video.rating * 0.5 : 0.0
                    starIconSize: 60
                }

                TextCustom {
                    id: details
                    anchors.left: parent.left
                    anchors.right: parent.right

                    fontSize: "small"
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                    color: "white"

                    property string baseText: u2d.tr('%1<br><br><b>Director:</b> %2<br><br><b>Cast:</b> %3')
                    text: (nfo.video) ? baseText.arg((nfo.video.plot) ? nfo.video.plot : "")
                                                .arg((nfo.video.director) ? nfo.video.director : "")
                                                .arg(nfo.getActors().join(", ")) : ""
                }
            }
        }

        AbstractScrollbar {
            id: scrollbar

            width: 5

            anchors.top: parent.top
            anchors.topMargin: Units.tvPx(15)
            anchors.bottom: parent.bottom
            anchors.bottomMargin: Units.tvPx(15)
            anchors.right: parent.right
            anchors.rightMargin: Units.tvPx(10)

            targetFlickable: infoFlickable
            /* The glow around the slider is 10 pixels wide. */
            sliderAnchors.rightMargin: -10
            sliderAnchors.leftMargin: -10
            sliderAnchors.topMargin: -10
            sliderAnchors.bottomMargin: -10

            sliderSmooth: false

            sliderSource: "../sidebar/artwork/scroll_bar.sci"

            /* Hide the scrollbar if there is less than a page of results */
            opacity: (targetFlickable.visibleArea.heightRatio < 1.0) && !targetFlickable.activeFocus ? 1.0 : 0.0
            Behavior on opacity { NumberAnimation { } }
        }

        AbstractScrollbar {
            id: glowingScrollbar

            width: 5

            anchors.fill: scrollbar

            targetFlickable: infoFlickable
            sliderAnchors.rightMargin: -10
            sliderAnchors.leftMargin: -10
            sliderAnchors.topMargin: -10
            sliderAnchors.bottomMargin: -10

            sliderSmooth: false

            sliderSource: "../sidebar/artwork/scroll_glow.sci"

            opacity: (targetFlickable.visibleArea.heightRatio < 1.0) && targetFlickable.activeFocus ? 1.0 : 0.0
            Behavior on opacity { NumberAnimation { } }
        }
    }
}
